@@ -1,9 +1,10 @@
-"""Default execution entry point if running the package via python -m"""
+"""Default execution entry point if running the package via python -m."""
 import pypyr.cli
 import sys
 
 
 def main():
+    """Run pypyr from script entry point."""
     return pypyr.cli.main()
 
 
