@@ -11,6 +11,7 @@ from pypyr.errors import (Call,
                           PipelineDefinitionError,
                           Stop)
 from pypyr.cache.stepcache import step_cache
+from pypyr.cache.backoffcache import backoff_cache
 from pypyr.utils import poll
 
 # use pypyr logger to ensure loglevel is set correctly
@@ -260,8 +261,6 @@ class Step:
                        to add to context before step execution.
         run_me: (bool) defaults True. step runs if this is true.
         skip_me: (bool) defaults False. step does not run if this is true.
-        steps_runner: pypyr.stepsrunner.StepsRunner Step Runner instance
-                      running this step.
         swallow_me: (bool) defaults False. swallow any errors during step run
                     and continue processing if true.
         while_decorator: (WhileDecorator) defaults None. execute step in while
@@ -269,7 +268,7 @@ class Step:
 
     """
 
-    def __init__(self, step, steps_runner):
+    def __init__(self, step):
         """Initialize the class. No duh, huh?.
 
         You can happily expect the initializer to initialize all
@@ -279,13 +278,9 @@ class Step:
             step: a string or a dict. This is the actual step as it exists in
                   the pipeline yaml - which is to say it can just be a string
                   for a simple step, or a dict for a complex step.
-            steps_runner: the StepsRunner instance running this Step.
-
         """
         logger.debug("starting")
 
-        self.steps_runner = steps_runner
-
         # defaults for decorators
         self.description = None
         self.foreach_items = None
@@ -474,8 +469,9 @@ class Step:
             self.run_step_function(context)
         except Call as call:
             logger.debug("call: calling %s", call.groups)
+            steps_runner = context.current_pipeline.steps_runner
             try:
-                self.steps_runner.run_step_groups(
+                steps_runner.run_step_groups(
                     groups=call.groups,
                     success_group=call.success_group,
                     failure_group=call.failure_group)
@@ -561,11 +557,11 @@ class Step:
         # change whether they evaluate True or False, thus apply formatting at
         # last possible instant.
         run_me = context.get_formatted_as_type(self.run_me, out_type=bool)
-        skip_me = context.get_formatted_as_type(self.skip_me, out_type=bool)
-        swallow_me = context.get_formatted_as_type(self.swallow_me,
-                                                   out_type=bool)
 
         if run_me:
+            skip_me = context.get_formatted_as_type(self.skip_me,
+                                                    out_type=bool)
+
             if not skip_me:
                 try:
                     if self.retry_decorator:
@@ -578,6 +574,9 @@ class Step:
                     # else, not errors per se.
                     raise
                 except Exception as exc_info:
+                    swallow_me = context.get_formatted_as_type(
+                        self.swallow_me, out_type=bool)
+
                     if isinstance(exc_info, HandledError):
                         exc_info = exc_info.__cause__
                     else:
@@ -605,6 +604,7 @@ class Step:
                             logger.error(
                                 "Error while running step %s", self.name
                             )
+
                         raise exc_info
             else:
                 logger.info(
@@ -640,21 +640,25 @@ class Step:
         """
         logger.debug("starting")
 
+        # the in params should be added to context before step execution.
+        self.set_step_input_context(context)
+
         # give user helpful output if step will actually run or not.
         if self.description:
             description = context.get_formatted_value(self.description)
             run_me = context.get_formatted_as_type(self.run_me, out_type=bool)
-            skip_me = context.get_formatted_as_type(self.skip_me,
-                                                    out_type=bool)
+
+            skip_me = False
+
+            if run_me:
+                skip_me = context.get_formatted_as_type(self.skip_me,
+                                                        out_type=bool)
 
             if run_me and not skip_me:
                 logger.notify(description)
             else:
                 logger.notify("(skipping): %s", description)
 
-        # the in params should be added to context before step execution.
-        self.set_step_input_context(context)
-
         if self.while_decorator:
             self.while_decorator.while_loop(context,
                                             self.run_foreach_or_conditional)
@@ -734,13 +738,25 @@ class RetryDecorator:
     retry_loop serves as the blackbox entrypoint for this class' other methods.
 
     Attributes:
-        max: (int) default None. Maximum loop iterations. None is infinite.
-        sleep: (float) defaults 0. Sleep in seconds between iterations.
-        stop_on: (list) default None. Always stop retry on these error
-                 types. None means retry on all errors.
-        retry_on: (list) default None. Only retry on these error types. All
-                other error types will stop retry loop. None means retry all
-                errors.
+        backoff (str): default 'fixed'. Absolute name of back-off strategy.
+            Builtin strategies allow aliases like fixed, linear, jitter. Custom
+            backoff should give absolute name to callable derived from
+            pypyr.retries.BackoffBase.
+        backoff_args (any): User provided arguments for back-off strategy.
+            Likely want to use a dict here.
+        jrc (float): default 0. Jitter Range Coefficient. Jitter finds a random
+            value between (jrc*sleep) and (sleep).
+        max (int):  default None. Maximum loop iterations. None is infinite.
+        sleep (float or list[float]):  defaults 0. Sleep in seconds between
+            iterations.
+        sleep_max (float): default None. Maximum value for sleep if using a
+            backoff strategy that calculates sleep interval. None means sleep
+            can increase indefinitely.
+        stop_on (list[str]): default None. Always stop retry on these error
+            types. None means retry on all errors.
+        retry_on (list[str]): default None. Only retry on these error types.
+            All other error types will stop retry loop. None means retry all
+            errors.
 
     """
 
@@ -758,12 +774,24 @@ class RetryDecorator:
         logger.debug("starting")
 
         if isinstance(retry_definition, dict):
+            # backoff: optional. defaults 'fixed' - set in retry_loop().
+            self.backoff = retry_definition.get('backoff', None)
+
+            # backoffArgs: optional.
+            self.backoff_args = retry_definition.get('backoffArgs', None)
+
+            # jrc: optional. defaults 0.
+            self.jrc = retry_definition.get('jrc', 0)
+
             # max: optional. defaults None.
             self.max = retry_definition.get('max', None)
 
             # sleep: optional. defaults 0.
             self.sleep = retry_definition.get('sleep', 0)
 
+            # sleep_max: optional. defaults None.
+            self.sleep_max = retry_definition.get('sleepMax', None)
+
             # stopOn: optional. defaults None.
             self.stop_on = retry_definition.get('stopOn', None)
 
@@ -874,30 +902,46 @@ class RetryDecorator:
         context['retryCounter'] = 0
         self.retry_counter = 0
 
-        sleep = context.get_formatted_as_type(self.sleep, out_type=float)
+        sleep = context.get_formatted_value(self.sleep)
+        backoff_name = context.get_formatted_value(
+            self.backoff) if self.backoff else 'fixed'
+
+        max_sleep = None
+        if self.sleep_max:
+            max_sleep = context.get_formatted_as_type(self.sleep_max,
+                                                      out_type=float)
+
+        jrc = context.get_formatted_value(self.jrc)
+        backoff_args = context.get_formatted_value(self.backoff_args)
+
+        backoff_callable = backoff_cache.get_backoff(backoff_name)(
+            sleep=sleep,
+            max_sleep=max_sleep,
+            jrc=jrc,
+            kwargs=backoff_args)
+
         if self.max:
             max = context.get_formatted_as_type(self.max, out_type=int)
 
-            logger.info("retry decorator will try %d times at %ss "
-                        "intervals.", max, sleep)
+            logger.info(
+                "retry decorator will try %d times with %s backoff starting "
+                "at %ss intervals.", max, backoff_name, sleep)
         else:
             max = None
-            logger.info("retry decorator will try indefinitely at %ss "
-                        "intervals.", sleep)
+            logger.info(
+                "retry decorator will try indefinitely with %s backoff "
+                "starting at %ss intervals.", backoff_name, sleep)
 
-        # this will never be false. because on counter == max,
-        # exec_iteration raises an exception, breaking out of the loop.
-        # pragma because cov doesn't know the implied else is impossible.
-        # unit test cov is 100%, though.
-        if poll.while_until_true(interval=sleep,
-                                 max_attempts=max)(
-                self.exec_iteration)(context=context,
-                                     step_method=step_method,
-                                     max=max
-                                     ):  # pragma: no cover
-            logger.debug("retry loop complete, reporting success.")
-
-        logger.debug("retry loop done")
+        # this will never be false. because on (counter == max) exec_iteration
+        # raises an exception, breaking out of the loop.
+        is_retry_ok = poll.while_until_true(interval=backoff_callable,
+                                            max_attempts=max)(
+            self.exec_iteration)(context=context,
+                                 step_method=step_method,
+                                 max=max
+                                 )
+        assert is_retry_ok
+        logger.debug("retry loop complete, reporting success.")
 
         logger.debug("done")
 
