@@ -1,14 +1,14 @@
 """Context parser that returns a dictionary from a json string."""
-
-import logging
+from collections.abc import Mapping
 import json
+import logging
 
-# use pypyrlogger to ensure loglevel is set correctly
 logger = logging.getLogger(__name__)
 
 
 def get_parsed_context(args):
     """Parse input context args and returns context as dictionary."""
+    logger.debug("starting")
     if not args:
         logger.debug("pipeline invoked without context arg set. For "
                      "this json parser you're looking for something "
@@ -17,6 +17,13 @@ def get_parsed_context(args):
                      "\"key2\":\"value2\"}'")
         return None
 
-    logger.debug("starting")
     # deserialize the input context string into json
-    return json.loads(' '.join(args))
+    payload = json.loads(' '.join(args))
+
+    if not isinstance(payload, Mapping):
+        raise TypeError("json input should describe an object at the top "
+                        "level. You should have something like \n"
+                        "{\n\"key1\":\"value1\",\n\"key2\":\"value2\"\n}\n"
+                        "at the json top-level, not an [array] or literal.")
+
+    return payload
